@@ -991,7 +991,7 @@ int freenet_request_stream (freenet_transfer_state *state, char *address,
 {
   freenet_message request;
   freenet_message reply;
-  int i, j, len, metadlen;
+  int i, j, len, metadlen, startpos;
   int depth;
   unsigned char header[FN_KEY_BYTES+2];
   int status;
@@ -1073,14 +1073,24 @@ int freenet_request_stream (freenet_transfer_state *state, char *address,
       } else if (strncmp(reply.field[i], "Storable.Signature=", 19)==0) {
         strncpy(state->sigstr, &(reply.field[i][19]), ((FN_DHK_LEN*2) + 2));
       } else if (strncmp(reply.field[i], "Storable.Public-key=", 20)==0) {
-        strncpy(pubstr, &(reply.field[i][20]), FN_DHK_LEN * 2);
+
+        /* skip some leading zeros.  this really sucks. */
+        startpos = 20;
+        if ((strlen(&(reply.field[i][startpos])) > FN_DHK_LEN * 2)
+            && ((startpos+1) < strlen(reply.field[i]))
+            && (reply.field[i][startpos] == '0')
+            && (reply.field[i][startpos+1] == '0')) {
+          startpos += 2;
+        }
+
+        strncpy(pubstr, &(reply.field[i][startpos]), FN_DHK_LEN * 2);
         pubstr[(FN_DHK_LEN*2) + 1] = 0;
         for (j = 0; j < strlen(pubstr) / 2; j++) {
           strncpy(tmp, &(pubstr[j*2]), 2);
           tmp[2] = 0;
           state->pub[j] = (unsigned char)strtol(tmp, NULL, 16);
         }
-      } else if (strncmp(reply.field[i], "Storable.Document-name=", 20)==0) {
+      } else if (strncmp(reply.field[i], "Storable.Document-name=", 23)==0) {
         strncpy(dnstr, &(reply.field[i][23]), FN_HASH_BYTES * 2);
         dnstr[(FN_HASH_BYTES*2) + 1] = 0;
         for (j = 0; j < strlen(dnstr) / 2; j++) {
